@@ -64,7 +64,7 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		}
 		#endregion
 		
-		#region IVisitor Members
+		#region IAstVisitor Members
 
 		override public void OnModule(Module m)
 		{
@@ -82,12 +82,21 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				WriteLine();
 			}
 
-			// m.Globals iria causar um Indent()
-			// invlido
 			if (null != m.Globals)
 			{
 				Visit(m.Globals.Statements);
 			}
+
+			foreach (Boo.Lang.Compiler.Ast.Attribute attribute in m.AssemblyAttributes)
+			{
+				WriteAssemblyAttribute(attribute);
+			}
+		}
+
+		private void WriteAssemblyAttribute(Attribute attribute)
+		{
+			WriteAttribute(attribute, "assembly: ");
+			WriteLine();
 		}
 
 		override public void OnNamespaceDeclaration(NamespaceDeclaration node)
@@ -149,6 +158,11 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			Dedent();
 		}
 		
+		override public void OnAttribute(Attribute att)
+		{
+			WriteAttribute(att);
+		}
+		
 		override public void OnClassDefinition(ClassDefinition c)
 		{
 			WriteTypeDefinition("class", c);
@@ -332,7 +346,12 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		}
 
 		override public void OnMethod(Method m)
-		{	
+		{
+            if (m.IsRuntime)
+            {
+                WriteIndented("// runtime");
+                WriteLine();
+            }
 			WriteCallableDefinitionHeader("def ", m);
 			WriteLine(":");
 			WriteBlock(m.Body);
@@ -350,8 +369,28 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		override public void OnParameterDeclaration(ParameterDeclaration p)
 		{
 			WriteAttributes(p.Attributes, false);
-			Write(p.Name);
-			WriteTypeReference(p.Type);
+			
+			if (p.IsByRef)
+			{
+				WriteKeyword("ref ");
+			}
+			
+			if (p.ParentNode.NodeType == NodeType.CallableTypeReference)
+			{
+				Visit(p.Type);
+			}
+			else
+			{
+				Write(p.Name);
+				WriteTypeReference(p.Type);
+			}
+		}
+		
+		override public void OnTypeofExpression(TypeofExpression node)
+		{
+			Write("typeof(");
+			Visit(node.Type);
+			Write(")");
 		}
 
 		override public void OnSimpleTypeReference(SimpleTypeReference t)
@@ -385,7 +424,7 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			Write(".");
 			Write(e.Name);
 		}
-		
+
 		override public void OnAsExpression(AsExpression e)
 		{
 			Write("(");
@@ -438,7 +477,8 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		
 		override public void OnUnaryExpression(UnaryExpression node)
 		{
-			bool addParens = NodeType.ExpressionStatement != node.ParentNode.NodeType;
+			bool addParens = NodeType.ExpressionStatement != node.ParentNode.NodeType
+				&& !IsMethodInvocationArg(node);
 			if (addParens)
 			{
 				Write("(");
@@ -460,6 +500,12 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			}
 		}
 
+		private bool IsMethodInvocationArg(UnaryExpression node)
+		{
+			MethodInvocationExpression parent = node.ParentNode as MethodInvocationExpression;
+			return null != parent && node != parent.Target;
+		}
+
 		override public void OnBinaryExpression(BinaryExpression e)
 		{
 			bool needsParens = !(e.ParentNode is ExpressionStatement);
@@ -594,6 +640,25 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			}
 			Write(e.Value);
 		}
+		
+		override public void OnAstLiteralExpression(AstLiteralExpression e)
+		{
+			WriteIndented();
+			WriteKeyword("ast");
+			if (e.Node is Expression)
+			{
+				Write(" { ");
+				Visit(e.Node);
+				Write(" }");
+			}
+			else
+			{
+				WriteLine(":");
+				Indent();
+				Visit(e.Node);
+				Dedent();
+			}
+		}
 
 		override public void OnStringLiteralExpression(StringLiteralExpression e)
 		{
@@ -620,6 +685,10 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		override public void OnDoubleLiteralExpression(DoubleLiteralExpression e)
 		{
 			Write(e.Value.ToString("########0.0##########", CultureInfo.InvariantCulture));
+			if (e.IsSingle)
+			{
+				Write("F");
+			}
 		}
 
 		override public void OnReferenceExpression(ReferenceExpression node)
@@ -854,6 +923,10 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 		{
 			switch (op)
 			{
+				case UnaryOperatorType.Explode:
+				{
+					return "*";
+				}
 				case UnaryOperatorType.PostIncrement:
 				case UnaryOperatorType.Increment:
 				{
@@ -875,6 +948,11 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 				{
 					return "not ";
 				}
+
+				case UnaryOperatorType.OnesComplement:
+				{
+					return "~";
+				}
 			}
 			throw new ArgumentException("op");
 		}
@@ -1232,28 +1310,42 @@ namespace Boo.Lang.Compiler.Ast.Visitors
 			Write(")");
 		}
 		
+		void WriteAttribute(Attribute attribute)
+		{
+			WriteAttribute(attribute, null);
+		}
+
+		void WriteAttribute(Attribute attribute, string prefix)
+		{
+			WriteIndented("[");
+			if (null != prefix)
+			{
+				Write(prefix);
+			}
+			Write(attribute.Name);
+			if (attribute.Arguments.Count > 0 ||
+			    attribute.NamedArguments.Count > 0)
+			{
+				Write("(");
+				WriteCommaSeparatedList(attribute.Arguments);
+				if (attribute.NamedArguments.Count > 0)
+				{
+					if (attribute.Arguments.Count > 0)
+					{
+						Write(", ");
+					}
+					WriteCommaSeparatedList(attribute.NamedArguments);
+				}
+				Write(")");
+			}
+			Write("]");
+		}
+		
 		void WriteAttributes(AttributeCollection attributes, bool addNewLines)
 		{
 			foreach (Boo.Lang.Compiler.Ast.Attribute attribute in attributes)
 			{
-				WriteIndented("[");
-				Write(attribute.Name);
-				if (attribute.Arguments.Count > 0 ||
-				    attribute.NamedArguments.Count > 0)
-				{
-					Write("(");
-					WriteCommaSeparatedList(attribute.Arguments);
-					if (attribute.NamedArguments.Count > 0)
-					{
-						if (attribute.Arguments.Count > 0)
-						{
-							Write(", ");
-						}
-						WriteCommaSeparatedList(attribute.NamedArguments);
-					}
-					Write(")");
-				}
-				Write("]");
+				Visit(attribute);
 				if (addNewLines)
 				{
 					WriteLine();
