@@ -145,10 +145,10 @@ namespace Boo.Lang.Compiler
 		{
 			return new CompilerError("BCE0019", node.LexicalInfo, node.Name, namespace_);
 		}
-		
-		public static CompilerError MemberNeedsInstance(Node node, string memberName)
+
+		public static CompilerError InstanceRequired(Node node, string typeName, string memberName)
 		{
-			return new CompilerError("BCE0020", node.LexicalInfo, memberName);
+			return new CompilerError("BCE0020", node.LexicalInfo, typeName, memberName);
 		}
 		
 		public static CompilerError InvalidNamespace(Import import)
@@ -320,7 +320,7 @@ namespace Boo.Lang.Compiler
 		{
 			return new CompilerError("BCE0057", node.LexicalInfo, name);
 		}
-		
+
 		public static CompilerError ObjectRequired(Node node)
 		{
 			return new CompilerError("BCE0058", node.LexicalInfo);
@@ -381,9 +381,9 @@ namespace Boo.Lang.Compiler
 			return new CompilerError("BCE0069", node.LexicalInfo, interfaceName, baseType);
 		}
 		
-		public static CompilerError RecursiveMethodWithoutReturnType(Node node)
+		public static CompilerError UnresolvedDependency(Node node, string source, string target)
 		{
-			return new CompilerError("BCE0070", node.LexicalInfo);
+			return new CompilerError("BCE0070", node.LexicalInfo, source, target);
 		}
 		
 		public static CompilerError InheritanceCycle(Node node, string typeName)
@@ -620,6 +620,61 @@ namespace Boo.Lang.Compiler
 		{
 			return new CompilerError("BCE0117", node.LexicalInfo, name);
 		}
+
+		public static CompilerError ExplodedExpressionMustBeArray(Node node)
+		{
+			return new CompilerError("BCE0118", node.LexicalInfo);
+		}
+
+		public static CompilerError ExplodeExpressionMustMatchVarArgCall(Node node)
+		{
+			return new CompilerError("BCE0119", node.LexicalInfo);
+		}
+
+		public static CompilerError UnaccessibleMember(Node node, string name)
+		{
+			return new CompilerError("BCE0120", node.LexicalInfo, name);
+		}
+
+		public static CompilerError InvalidSuper(Node node)
+		{
+			return new CompilerError("BCE0121", node.LexicalInfo);
+		}
+
+		public static CompilerError ValueTypeCantHaveAbstractMember(Node node, string typeName, string memberName)
+		{
+			return new CompilerError("BCE0122", node.LexicalInfo, typeName, memberName);
+		}
+
+		public static CompilerError InvalidParameterType(Node node, string typeName)
+		{
+			return new CompilerError("BCE0123", node.LexicalInfo, typeName);
+		}
+
+		public static CompilerError InvalidFieldType(Node node, string typeName)
+		{
+			return new CompilerError("BCE0124", node.LexicalInfo, typeName);
+		}
+
+		public static CompilerError InvalidDeclarationType(Node node, string typeName)
+		{
+			return new CompilerError("BCE0125", node.LexicalInfo, typeName);
+		}
+
+		public static CompilerError InvalidExpressionType(Node node, string typeName)
+		{
+			return new CompilerError("BCE0126", node.LexicalInfo, typeName);
+		}
+		
+		public static CompilerError RefArgTakesLValue(Node node)
+		{
+			return new CompilerError("BCE0127", node.LexicalInfo, node.ToString());
+		}
+
+		public static CompilerError InvalidTryStatement(Node node)
+		{
+			return new CompilerError("BCE0128", node.LexicalInfo);
+		}
 		
 		public static string ToStringList(System.Collections.IEnumerable names)
 		{
