@@ -40,6 +40,7 @@ using Boo.Lang.Runtime;
 using Boo.Lang.Compiler.Ast;
 using Boo.Lang.Compiler;
 using Boo.Lang.Compiler.TypeSystem;
+using Module = Boo.Lang.Compiler.Ast.Module;
 
 namespace Boo.Lang.Compiler.Steps
 {
@@ -62,8 +63,10 @@ namespace Boo.Lang.Compiler.Steps
 	public class EmitAssembly : AbstractVisitorCompilerStep
 	{
 		static ConstructorInfo DebuggableAttribute_Constructor = typeof(System.Diagnostics.DebuggableAttribute).GetConstructor(new Type[] { Types.Bool, Types.Bool });
+
+		static ConstructorInfo DuckTypedAttribute_Constructor = Types.DuckTypedAttribute.GetConstructor(new Type[0]);
 		
-		static ConstructorInfo ParamArrayAttribute_Constructor = typeof(System.ParamArrayAttribute).GetConstructor(new Type[0]);
+		static ConstructorInfo ParamArrayAttribute_Constructor = Types.ParamArrayAttribute.GetConstructor(new Type[0]);
 		
 		static MethodInfo RuntimeServices_NormalizeArrayIndex = Types.RuntimeServices.GetMethod("NormalizeArrayIndex");
 		
@@ -104,7 +107,7 @@ namespace Boo.Lang.Compiler.Steps
 		IType _returnType;
 		int _tryBlock; // are we in a try block?
 		bool _checked = true;
-		bool _rawarrayindexing = false;
+		bool _rawArrayIndexing = false;
 		Hashtable _typeCache = new Hashtable();
 		
 		// keeps track of types on the IL stack
@@ -387,7 +390,7 @@ namespace Boo.Lang.Compiler.Steps
 			_returnType = null;
 			_tryBlock = 0;
 			_checked = true;
-			_rawarrayindexing = false;
+			_rawArrayIndexing = false;
 			_types.Clear();
 			_typeCache.Clear();
 			_builders.Clear();
@@ -400,22 +403,28 @@ namespace Boo.Lang.Compiler.Steps
 		
 		override public void OnModule(Boo.Lang.Compiler.Ast.Module module)
 		{
+			InitializeDebugInfoWriter(module);
+			Visit(module.Members);
+		}
+
+		private void InitializeDebugInfoWriter(Module module)
+		{
+			if (!Parameters.Debug) return;
 			string fname = module.LexicalInfo.FileName;
 			if (null != fname)
 			{
 				_symbolDocWriter = _moduleBuilder.DefineDocument(
-										fname,
-										Guid.Empty,
-										Guid.Empty,
-										SymDocumentType.Text);
+					fname,
+					Guid.Empty,
+					Guid.Empty,
+					SymDocumentType.Text);
 			}
 			else
 			{
 				_symbolDocWriter = null;
 			}
-			Visit(module.Members);
 		}
-		
+
 		override public void OnEnumDefinition(EnumDefinition node)
 		{
 			Type baseType = typeof(int);
@@ -486,15 +495,14 @@ namespace Boo.Lang.Compiler.Steps
 			}
 			
 			MethodBuilder methodBuilder = GetMethodBuilder(method);
-
 			if (null != method.ExplicitInfo)
 			{
 				IMethod ifaceMethod = (IMethod)method.ExplicitInfo.Entity;
 				MethodInfo ifaceInfo = GetMethodInfo(ifaceMethod);
-				MethodInfo implInfo = GetMethodInfo ((IMethod)method.Entity);
+				MethodInfo implInfo = GetMethodInfo((IMethod)method.Entity);
 
 				TypeBuilder typeBuilder = GetTypeBuilder((ClassDefinition)method.DeclaringType);
-				typeBuilder.DefineMethodOverride (implInfo, ifaceInfo);
+				typeBuilder.DefineMethodOverride(implInfo, ifaceInfo);
 			}
 
 			_il = methodBuilder.GetILGenerator();
@@ -505,7 +513,7 @@ namespace Boo.Lang.Compiler.Steps
 			{
 				_returnValueLocal = _il.DeclareLocal(GetSystemType(_returnType));
 			}
-			
+
 			DefineLabels(method);
 			Visit(method.Locals);
 			Visit(method.Body);
@@ -519,7 +527,7 @@ namespace Boo.Lang.Compiler.Steps
 			}
 			_il.Emit(OpCodes.Ret);
 		}
-		
+
 		override public void OnBlock(Block block)
 		{
 			bool current = _checked;
@@ -530,18 +538,18 @@ namespace Boo.Lang.Compiler.Steps
 				_checked = (bool)objChecked;
 			}
 			
-			bool current_indexing = _rawarrayindexing;
+			bool currentArrayIndexing = _rawArrayIndexing;
 			object objRawArrayIndexing = block["rawarrayindexing"];
 			
 			if (objRawArrayIndexing is bool)
 			{
-				_rawarrayindexing = (bool)objRawArrayIndexing;
+				_rawArrayIndexing = (bool)objRawArrayIndexing;
 			}
 
 			Visit(block.Statements);
 
 			_checked = current;
-			_rawarrayindexing = current_indexing;
+			_rawArrayIndexing = currentArrayIndexing;
 		}
 		
 		void DefineLabels(Method method)
@@ -607,8 +615,21 @@ namespace Boo.Lang.Compiler.Steps
 				Visit(node.Exception); PopType();
 				_il.Emit(OpCodes.Throw);
 			}
+			EmitNopDebugInfo(node);
 		}
-		
+
+		private void EmitNopDebugInfo(Statement node)
+		{
+			// HACK: workaround - mono reports the position of
+			// raise as being the position of the next instruction
+			// after it
+			if (CanEmitDebugInfo())
+			{
+				EmitDebugInfo(node);
+				_il.Emit(OpCodes.Nop);
+			}
+		}
+
 		override public void OnTryStatement(TryStatement node)
 		{
 			++_tryBlock;
@@ -651,6 +672,8 @@ namespace Boo.Lang.Compiler.Steps
 			// the stack sane
 			DiscardValueOnStack();
 			AssertStackIsEmpty("stack must be empty after a statement!");
+
+			EmitNopDebugInfo(node);
 		}
 		
 		void DiscardValueOnStack()
@@ -1002,28 +1025,19 @@ namespace Boo.Lang.Compiler.Steps
 			{
 				case UnaryOperatorType.LogicalNot:
 				{
-					node.Operand.Accept(this);
-					IType typeOnStack = PopType();
-					if (IsBoolOrInt(typeOnStack) || EmitToBoolIfNeeded(typeOnStack))
-					{
-						EmitIntNot();
-					}
-					else
-					{
-						EmitGenericNot();
-					}
-					PushBool();
+					EmitLogicalNot(node);
 					break;
 				}
 				
 				case UnaryOperatorType.UnaryNegation:
 				{
-					node.Operand.Accept(this);
-					IType type = PopType();
-					_il.Emit(OpCodes.Ldc_I4, -1);
-					EmitCastIfNeeded(type, TypeSystemServices.IntType);
-					_il.Emit(OpCodes.Mul);
-					PushType(type);
+					EmitUnaryNegation(node);
+					break;
+				}
+
+				case UnaryOperatorType.OnesComplement:
+				{
+					EmitOnesComplement(node);
 					break;
 				}
 				
@@ -1034,7 +1048,38 @@ namespace Boo.Lang.Compiler.Steps
 				}
 			}
 		}
-		
+
+		private void EmitOnesComplement(UnaryExpression node)
+		{
+			node.Operand.Accept(this);
+			_il.Emit(OpCodes.Not);
+		}
+
+		private void EmitLogicalNot(UnaryExpression node)
+		{
+			node.Operand.Accept(this);
+			IType typeOnStack = PopType();
+			if (IsBoolOrInt(typeOnStack) || EmitToBoolIfNeeded(typeOnStack))
+			{
+				EmitIntNot();
+			}
+			else
+			{
+				EmitGenericNot();
+			}
+			PushBool();
+		}
+
+		private void EmitUnaryNegation(UnaryExpression node)
+		{
+			node.Operand.Accept(this);
+			IType type = PopType();
+			_il.Emit(OpCodes.Ldc_I4, -1);
+			EmitCastIfNeeded(type, TypeSystemServices.IntType);
+			_il.Emit(OpCodes.Mul);
+			PushType(type);
+		}
+
 		bool ShouldLeaveValueOnStack(Expression node)
 		{
 			return node.ParentNode.NodeType != NodeType.ExpressionStatement;
@@ -1126,6 +1171,11 @@ namespace Boo.Lang.Compiler.Steps
 				{
 					InternalParameter param = (InternalParameter)tag;
 					
+					if (param.Parameter.IsByRef)
+					{
+						LoadParam(param);
+					}
+					
 					Visit(node.Right);
 					EmitCastIfNeeded(param.Type, PopType());
 					
@@ -1134,7 +1184,23 @@ namespace Boo.Lang.Compiler.Steps
 						_il.Emit(OpCodes.Dup);
 						PushType(param.Type);
 					}
-					_il.Emit(OpCodes.Starg, param.Index);
+					
+					if (param.Parameter.IsByRef)
+					{
+						OpCode storecode = GetStoreRefParamCode(param.Type);
+						if (IsStobj(storecode)) //passing struct/decimal byref
+						{
+							_il.Emit(storecode, GetSystemType(param.Type));
+						}
+						else
+						{
+							_il.Emit(storecode);
+						}
+					}
+					else
+					{
+						_il.Emit(OpCodes.Starg, param.Index);
+					}
 					break;
 				}
 				
@@ -1570,44 +1636,60 @@ namespace Boo.Lang.Compiler.Steps
 		
 		void InvokeRegularMethod(IMethod method, MethodInfo mi, MethodInvocationExpression node)
 		{
-			OpCode code = OpCodes.Call;
 			if (!mi.IsStatic)
 			{
-				Expression target = ((MemberReferenceExpression)node.Target).Target;
-				IType targetType = target.ExpressionType;
-				if (targetType.IsValueType)
-				{
-					if (mi.DeclaringType.IsValueType)
-					{
-						LoadAddress(target);
-					}
-					else
-					{
-						Visit(node.Target);
-						EmitBox(PopType());
-						code = OpCodes.Callvirt;
-					}
-				}
-				else
-				{
-					code = OpCodes.Callvirt;
-					
-					// pushes target reference
-					Visit(node.Target); PopType();
-				}
-				
-				if (NodeType.SuperLiteralExpression == target.NodeType)
-				{
-					code = OpCodes.Call;
-				}
+				PushTargetObject(node, mi);
 			}
 			
 			PushArguments(method, node.Arguments);
-			_il.EmitCall(code, mi, null);
+			_il.EmitCall(GetCallOpCode(node, method, mi), mi, null);
 			
 			PushType(method.ReturnType);
 		}
-		
+
+		private void PushTargetObject(MethodInvocationExpression node, MethodInfo mi)
+		{
+			Expression target = GetTargetObject(node);
+			IType targetType = target.ExpressionType;
+			if (targetType.IsValueType)
+			{
+				if (mi.DeclaringType.IsValueType)
+				{
+					LoadAddress(target);
+				}
+				else
+				{
+					Visit(node.Target);
+					EmitBox(PopType());
+				}
+			}
+			else
+			{	
+				// pushes target reference
+				Visit(node.Target);
+				PopType();
+			}
+		}
+
+		private static Expression GetTargetObject(MethodInvocationExpression node)
+		{
+			return ((MemberReferenceExpression)node.Target).Target;
+		}
+
+		private OpCode GetCallOpCode(MethodInvocationExpression node, IMethod method, MethodInfo mi)
+		{
+			if (method.IsStatic) return OpCodes.Call;
+			if (NodeType.SuperLiteralExpression == GetTargetObject(node).NodeType) return OpCodes.Call;
+			if (IsValueTypeMethodCall(node, method)) return OpCodes.Call;
+			return OpCodes.Callvirt;
+		}
+
+		private bool IsValueTypeMethodCall(MethodInvocationExpression node, IMethod method)
+		{
+			IType type = GetTargetObject(node).ExpressionType;
+			return type.IsValueType && method.DeclaringType == type;
+		}
+
 		void InvokeSuperMethod(IMethod methodInfo, MethodInvocationExpression node)
 		{
 			IMethod super = ((InternalMethod)methodInfo).Overriden;
@@ -1840,13 +1922,15 @@ namespace Boo.Lang.Compiler.Steps
 			foreach (ExpressionPair pair in node.Items)
 			{
 				_il.Emit(OpCodes.Dup);
+				
 				Visit(pair.First);
 				EmitCastIfNeeded(objType, PopType());
 				
 				Visit(pair.Second);
 				EmitCastIfNeeded(objType, PopType());
-				_il.EmitCall(OpCodes.Call, Hash_Add, null);
+				_il.EmitCall(OpCodes.Callvirt, Hash_Add, null);
 			}
+			
 			PushType(TypeSystemServices.HashType);
 		}
 		
@@ -1926,7 +2010,7 @@ namespace Boo.Lang.Compiler.Steps
 		void EmitNormalizedArrayIndex(Expression index)
 		{
 			bool isNegative = false;
-			if (CanBeNegative(index, ref isNegative) && (!_rawarrayindexing))
+			if (CanBeNegative(index, ref isNegative) && (!_rawArrayIndexing))
 			{
 				if (isNegative)
 				{
@@ -2090,6 +2174,18 @@ namespace Boo.Lang.Compiler.Steps
 						break;
 					}
 					
+					case TypeCode.Int16:
+					{
+						_il.Emit(OpCodes.Ldc_I4, (int)(short)value);
+						break;
+					}
+
+					case TypeCode.UInt16:
+					{
+						_il.Emit(OpCodes.Ldc_I4, (int)(ushort)value);
+						break;
+					}
+					
 					case TypeCode.Int32:
 					{
 						_il.Emit(OpCodes.Ldc_I4, (int)value);
@@ -2210,7 +2306,15 @@ namespace Boo.Lang.Compiler.Steps
 				
 					case EntityType.Parameter:
 					{
-						_il.Emit(OpCodes.Ldarga, ((InternalParameter)tag).Index);
+						InternalParameter param = (InternalParameter)tag;
+						if (param.Parameter.IsByRef)
+						{
+							LoadParam(param);
+						}
+						else
+						{
+							_il.Emit(OpCodes.Ldarga, param.Index);
+						}
 						return;
 					}
 					
@@ -2299,44 +2403,17 @@ namespace Boo.Lang.Compiler.Steps
 				case EntityType.Parameter:
 				{
 					TypeSystem.InternalParameter param = (TypeSystem.InternalParameter)info;
-					int index = param.Index;
-					switch (index)
+					LoadParam(param);
+					
+					if (param.Parameter.IsByRef)
 					{
-						case 0:
+						OpCode code = GetLoadRefParamCode(param.Type);
+						if (code.Value == OpCodes.Ldobj.Value)
 						{
-							_il.Emit(OpCodes.Ldarg_0);
-							break;
+							_il.Emit(code, GetSystemType(param.Type));
 						}
-						
-						case 1:
-						{
-							_il.Emit(OpCodes.Ldarg_1);
-							break;
-						}
-						
-						case 2:
-						{
-							_il.Emit(OpCodes.Ldarg_2);
-							break;
-						}
-						
-						case 3:
-						{
-							_il.Emit(OpCodes.Ldarg_3);
-							break;
-						}
-						
-						default:
-						{
-							if (index < 256)
-							{
-								_il.Emit(OpCodes.Ldarg_S, index);
-							}
-							else
-							{
-								_il.Emit(OpCodes.Ldarg, index);
-							}
-							break;
+						else {
+							_il.Emit(code);
 						}
 					}
 					PushType(param.Type);
@@ -2359,6 +2436,50 @@ namespace Boo.Lang.Compiler.Steps
 			}
 		}
 		
+		void LoadParam(TypeSystem.InternalParameter param)
+		{
+			int index = param.Index;
+			
+			switch (index)
+			{
+				case 0:
+				{
+					_il.Emit(OpCodes.Ldarg_0);
+					break;
+				}
+				
+				case 1:
+				{
+					_il.Emit(OpCodes.Ldarg_1);
+					break;
+				}
+				
+				case 2:
+				{
+					_il.Emit(OpCodes.Ldarg_2);
+					break;
+				}
+				
+				case 3:
+				{
+					_il.Emit(OpCodes.Ldarg_3);
+					break;
+				}
+				
+				default:
+				{
+					if (index < 256)
+					{
+						_il.Emit(OpCodes.Ldarg_S, index);
+					}
+					else
+					{
+						_il.Emit(OpCodes.Ldarg, index);
+					}
+					break;
+				}
+			}
+		}
 		void SetLocal(BinaryExpression node, InternalLocal tag, bool leaveValueOnStack)
 		{
 			node.Right.Accept(this); // leaves type on stack
@@ -2472,17 +2593,13 @@ namespace Boo.Lang.Compiler.Steps
 		
 		void EmitDebugInfo(Node startNode, Node endNode)
 		{
-			if (null == _symbolDocWriter)
-			{
-				return;
-			}
+			if (!CanEmitDebugInfo()) return;
 			
 			LexicalInfo start = startNode.LexicalInfo;
 			if (start.IsValid)
 			{
 				try
-				{
-					//_il.Emit(OpCodes.Nop);
+				{	
 					_il.MarkSequencePoint(_symbolDocWriter, start.Line, 0, start.Line+1, 0);
 				}
 				catch (Exception x)
@@ -2491,22 +2608,33 @@ namespace Boo.Lang.Compiler.Steps
 				}
 			}
 		}
-		
+
+		private bool CanEmitDebugInfo()
+		{
+			return null != _symbolDocWriter;
+		}
+
 		bool IsBoolOrInt(IType type)
 		{
 			return TypeSystemServices.BoolType == type ||
 				TypeSystemServices.IntType == type;
 		}
 		
-		void PushArguments(IMethod tag, ExpressionCollection args)
+		void PushArguments(IMethodBase entity, ExpressionCollection args)
 		{
-			IParameter[] parameters = tag.GetParameters();
+			IParameter[] parameters = entity.GetParameters();
 			for (int i=0; i<args.Count; ++i)
 			{
 				IType parameterType = parameters[i].Type;
-				
 				Expression arg = args[i];
-				if (parameterType.IsByRef)
+				/*
+				InternalParameter internalparam = parameters[i] as InternalParameter;
+				if ((parameterType.IsByRef) || 
+					(internalparam != null && 
+					internalparam.Parameter.IsByRef)
+					)
+				*/
+				if (parameters[i].IsByRef)
 				{
 					LoadAddress(arg);
 				}
@@ -2671,6 +2799,86 @@ namespace Boo.Lang.Compiler.Steps
 			return OpCodes.Stelem_Ref;
 		}
 		
+		OpCode GetLoadRefParamCode(IType tag)
+		{
+			if (tag.IsValueType)
+			{
+				if (TypeSystemServices.IntType == tag ||
+					tag.IsEnum)
+				{
+					return OpCodes.Ldind_I4;
+				}
+				if (TypeSystemServices.LongType == tag)
+				{
+					return OpCodes.Ldind_I8;
+				}
+				if (TypeSystemServices.ByteType == tag)
+				{
+					return OpCodes.Ldind_I1;
+				}
+				if (TypeSystemServices.ShortType == tag ||
+					TypeSystemServices.CharType == tag)
+				{
+					return OpCodes.Ldind_I2;
+				}
+				if (TypeSystemServices.SingleType == tag)
+				{
+					return OpCodes.Ldind_R4;
+				}
+				if (TypeSystemServices.DoubleType == tag)
+				{
+					return OpCodes.Ldind_R8;
+				}
+				if (TypeSystemServices.UShortType == tag)
+				{
+					return OpCodes.Ldind_U2;
+				}
+				if (TypeSystemServices.UIntType == tag)
+				{
+					return OpCodes.Ldind_U4;
+				}
+				
+				return OpCodes.Ldobj;
+			}
+			return OpCodes.Ldind_Ref;
+		}
+		
+		OpCode GetStoreRefParamCode(IType tag)
+		{
+			if (tag.IsValueType)
+			{
+				if (TypeSystemServices.IntType == tag ||
+					tag.IsEnum)
+				{
+					return OpCodes.Stind_I4;
+				}
+				if (TypeSystemServices.LongType == tag)
+				{
+					return OpCodes.Stind_I8;
+				}
+				if (TypeSystemServices.ByteType == tag)
+				{
+					return OpCodes.Stind_I1;
+				}
+				if (TypeSystemServices.ShortType == tag ||
+					TypeSystemServices.CharType == tag)
+				{
+					return OpCodes.Stind_I2;
+				}
+				if (TypeSystemServices.SingleType == tag)
+				{
+					return OpCodes.Stind_R4;
+				}
+				if (TypeSystemServices.DoubleType == tag)
+				{
+					return OpCodes.Stind_R8;
+				}
+				
+				return OpCodes.Stobj;
+			}
+			return OpCodes.Stind_Ref;
+		}
+		
 		void EmitCastIfNeeded(IType expectedType, IType actualType)
 		{
 			if (null == actualType) // see NullLiteralExpression
@@ -2727,9 +2935,10 @@ namespace Boo.Lang.Compiler.Steps
 		
 		void EmitUnbox(IType expectedType)
 		{
-			if (TypeSystemServices.IsNumberOrBool(expectedType))
+			string unboxMethodName = GetUnboxMethodName(expectedType);
+			if (null != unboxMethodName)
 			{
-				_il.EmitCall(OpCodes.Call, GetUnboxMethod(expectedType), null);
+				_il.EmitCall(OpCodes.Call, GetRuntimeMethod(unboxMethodName), null);
 			}
 			else
 			{
@@ -2739,9 +2948,9 @@ namespace Boo.Lang.Compiler.Steps
 			}
 		}
 		
-		MethodInfo GetUnboxMethod(IType type)
+		MethodInfo GetRuntimeMethod(string methodName)
 		{
-			return typeof(RuntimeServices).GetMethod(GetUnboxMethodName(type));
+			return Types.RuntimeServices.GetMethod(methodName);
 		}
 		
 		string GetUnboxMethodName(IType type)
@@ -2794,10 +3003,11 @@ namespace Boo.Lang.Compiler.Steps
 			{
 				return "UnboxBoolean";
 			}
-			else
+			else if (type == TypeSystemServices.CharType)
 			{
-				throw new NotImplementedException(string.Format("Numeric promotion for {0} not implemented!", type));
+				return "UnboxChar";
 			}
+			return null;
 		}
 		
 		OpCode GetNumericPromotionOpCode(IType type)
@@ -2883,18 +3093,13 @@ namespace Boo.Lang.Compiler.Steps
 			{
 				_asmBuilder.SetCustomAttribute(GetCustomAttributeBuilder(attribute));
 			}
-			
-			if (Parameters.Debug)
-			{
-				_asmBuilder.SetCustomAttribute(CreateDebuggableAttribute());
-			}
 		}
 		
 		CustomAttributeBuilder CreateDebuggableAttribute()
 		{
 			return new CustomAttributeBuilder(
 								DebuggableAttribute_Constructor,
-								new object[] { true, false });
+								new object[] { true, true });
 		}
 		
 		void DefineEntryPoint()
@@ -2927,6 +3132,19 @@ namespace Boo.Lang.Compiler.Steps
 			for (int i=0; i<types.Length; ++i)
 			{
 				types[i] = GetSystemType(parameters[i].Type);
+				if (parameters[i].IsByRef)
+				{
+					string typename = TypeSystemServices.GetReferenceTypeName(types[i]);
+					Type byreftype = types[i].Assembly.GetType(typename);
+					
+					if (byreftype == null) //internal type
+					{
+						byreftype = _moduleBuilder.GetType(typename,true);
+						//TODO ? - test that nested types work too
+						//GetTypeBuilder(parameters[i].Type).GetNestedType(typename);
+					}
+					types[i] = byreftype;
+				}
 			}
 			return types;
 		}
@@ -3243,6 +3461,10 @@ namespace Boo.Lang.Compiler.Steps
 			{
 				attributes |= MethodAttributes.Static;
 			}
+			if (member.IsFinal)
+			{
+				attributes |= MethodAttributes.Final;
+			}
 			if (member.IsAbstract)
 			{
 				attributes |= (MethodAttributes.Abstract | MethodAttributes.Virtual);
@@ -3315,6 +3537,11 @@ namespace Boo.Lang.Compiler.Steps
 			return attributes;
 		}
 		
+		ParameterAttributes GetParameterAttributes(ParameterDeclaration param)
+		{
+			return ParameterAttributes.None;
+		}
+		
 		void DefineEvent(TypeBuilder typeBuilder, Event node)
 		{
 			EventBuilder builder = typeBuilder.DefineEvent(node.Name,
@@ -3324,7 +3551,12 @@ namespace Boo.Lang.Compiler.Steps
 			MethodAttributes baseAttributes = MethodAttributes.SpecialName;
 			builder.SetAddOnMethod(DefineMethod(typeBuilder, node.Add, baseAttributes|GetMethodAttributes(node.Add)));
 			builder.SetRemoveOnMethod(DefineMethod(typeBuilder, node.Remove, baseAttributes|GetMethodAttributes(node.Remove)));
-			builder.SetRaiseMethod(DefineMethod(typeBuilder, node.Raise, baseAttributes|GetMethodAttributes(node.Raise)));
+
+			if (null != node.Raise)
+			{
+				builder.SetRaiseMethod(DefineMethod(typeBuilder, node.Raise, baseAttributes|GetMethodAttributes(node.Raise)));
+			}
+
 			SetBuilder(node, builder);
 		}
 		
@@ -3365,6 +3597,12 @@ namespace Boo.Lang.Compiler.Steps
 			{
 				builder.SetCustomAttribute(GetCustomAttributeBuilder(attribute));
 			}
+
+			bool isDuckTyped = GetEntity(property).IsDuckTyped;
+			if (isDuckTyped)
+			{
+				builder.SetCustomAttribute(CreateDuckTypedCustomAttribute());
+			}
 			
 			SetBuilder(property, builder);
 		}
@@ -3386,11 +3624,15 @@ namespace Boo.Lang.Compiler.Steps
 			int last = parameters.Count - 1;
 			for (int i=0; i<parameters.Count; ++i)
 			{
-				ParameterBuilder paramBuilder = builder.DefineParameter(i+1, ParameterAttributes.None, parameters[i].Name);
+				ParameterBuilder paramBuilder = builder.DefineParameter(i+1, GetParameterAttributes(parameters[i]), parameters[i].Name);
 				if (last == i && parameters.VariableNumber)
 				{
 					SetParamArrayAttribute(paramBuilder);
 				}
+				foreach (Boo.Lang.Compiler.Ast.Attribute attribute in parameters[i].Attributes)
+				{
+					paramBuilder.SetCustomAttribute(GetCustomAttributeBuilder(attribute));
+				}
 			}
 			
 		}
@@ -3400,11 +3642,15 @@ namespace Boo.Lang.Compiler.Steps
 			int last = parameters.Count - 1;
 			for (int i=0; i<parameters.Count; ++i)
 			{
-				ParameterBuilder paramBuilder = builder.DefineParameter(i+1, ParameterAttributes.None, parameters[i].Name);
+				ParameterBuilder paramBuilder = builder.DefineParameter(i+1, GetParameterAttributes(parameters[i]), parameters[i].Name);
 				if (last == i && parameters.VariableNumber)
 				{
 					SetParamArrayAttribute(paramBuilder);
 				}
+				foreach (Boo.Lang.Compiler.Ast.Attribute attribute in parameters[i].Attributes)
+				{
+					paramBuilder.SetCustomAttribute(GetCustomAttributeBuilder(attribute));
+				}
 			}
 		}
 		
@@ -3457,9 +3703,19 @@ namespace Boo.Lang.Compiler.Steps
 			{
 				builder.SetCustomAttribute(GetCustomAttributeBuilder(attribute));
 			}
+
+			if (GetEntity(method).IsDuckTyped)
+			{
+				builder.SetCustomAttribute(CreateDuckTypedCustomAttribute());
+			}
 			return builder;
 		}
-		
+
+		private CustomAttributeBuilder CreateDuckTypedCustomAttribute()
+		{
+			return new CustomAttributeBuilder(DuckTypedAttribute_Constructor, new object[0]);
+		}
+
 		void DefineConstructor(TypeBuilder typeBuilder, Method constructor)
 		{
 			ConstructorBuilder builder = typeBuilder.DefineConstructor(GetMethodAttributes(constructor),
@@ -3754,8 +4010,39 @@ namespace Boo.Lang.Compiler.Steps
 		{
 			foreach (ICompilerResource resource in Parameters.Resources)
 			{
-				IResourceWriter writer = _moduleBuilder.DefineResource(resource.Name, resource.Description);
-				resource.WriteResources(writer);
+				resource.WriteResource(_sreResourceService);
+			}
+		}
+
+		SREResourceService _sreResourceService;
+
+		class SREResourceService : IResourceService
+		{
+			AssemblyBuilder _asmBuilder;
+			ModuleBuilder _moduleBuilder;
+
+			public SREResourceService (AssemblyBuilder asmBuilder, ModuleBuilder modBuilder)
+			{
+				this._asmBuilder = asmBuilder;
+				this._moduleBuilder = modBuilder;
+			}
+
+			public bool EmbedFile(string resourceName, string fname)
+			{
+				MethodInfo embed_res = typeof (System.Reflection.Emit.AssemblyBuilder).GetMethod(
+					"EmbedResourceFile", BindingFlags.Instance|BindingFlags.Public|BindingFlags.NonPublic,
+					null, CallingConventions.Any, new Type[] { typeof(string), typeof(string) }, null);
+				if (embed_res != null)
+				{
+					embed_res.Invoke(this._asmBuilder, new object[] { resourceName, fname });
+					return true;
+				}
+				return false;
+			}
+
+			public IResourceWriter DefineResource(string resourceName, string resourceDescription)
+			{
+				return this._moduleBuilder.DefineResource(resourceName, resourceDescription);
 			}
 		}
 		
@@ -3771,7 +4058,15 @@ namespace Boo.Lang.Compiler.Steps
 			
 			AssemblyName asmName = CreateAssemblyName(outputFile);
 			_asmBuilder = AppDomain.CurrentDomain.DefineDynamicAssembly(asmName, GetAssemblyBuilderAccess(), GetTargetDirectory(outputFile));
+			if (Parameters.Debug)
+			{
+				// ikvm tip:  Set DebuggableAttribute to assembly before
+				// creating the module, to make sure Visual Studio (Whidbey)
+				// picks up the attribute when debugging dynamically generated code.
+				_asmBuilder.SetCustomAttribute(CreateDebuggableAttribute());
+			}
 			_moduleBuilder = _asmBuilder.DefineDynamicModule(asmName.Name, Path.GetFileName(outputFile), true);
+			_sreResourceService = new SREResourceService (_asmBuilder, _moduleBuilder);
 			ContextAnnotations.SetAssemblyBuilder(Context, _asmBuilder);
 			
 			Context.GeneratedAssemblyFileName = outputFile;
