@@ -26,6 +26,37 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
+
+using System;
+
+#region license
+// Copyright (c) 2003, 2004, 2005 Rodrigo B. de Oliveira (rbo@acm.org)
+// All rights reserved.
+// 
+// Redistribution and use in source and binary forms, with or without modification,
+// are permitted provided that the following conditions are met:
+// 
+//     * Redistributions of source code must retain the above copyright notice,
+//     this list of conditions and the following disclaimer.
+//     * Redistributions in binary form must reproduce the above copyright notice,
+//     this list of conditions and the following disclaimer in the documentation
+//     and/or other materials provided with the distribution.
+//     * Neither the name of Rodrigo B. de Oliveira nor the names of its
+//     contributors may be used to endorse or promote products derived from this
+//     software without specific prior written permission.
+// 
+// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
+// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
+// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
+// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
+// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+#endregion
+
 namespace Boo.Lang.Compiler.Steps
 {
 	using System.Diagnostics;
@@ -118,7 +149,7 @@ namespace Boo.Lang.Compiler.Steps
 			foreach (TypeReference baseTypeRef in node.BaseTypes)
 			{	
 				IType baseType = GetType(baseTypeRef);
-				EnsureRelatedNodeWasVisited(baseType);
+				EnsureRelatedNodeWasVisited(node, baseType);
 				if (baseType.IsInterface)
 				{
 					ResolveInterfaceMembers(node, baseTypeRef, baseType);
@@ -222,6 +253,41 @@ namespace Boo.Lang.Compiler.Steps
 			return p;
 		}
 		
+		void ResolveAbstractEvent(ClassDefinition node,
+			TypeReference baseTypeRef,
+			IEvent entity)
+		{
+			TypeMember member = node.Members[entity.Name];
+			if (null != member)
+			{
+				Event ev = (Event)member;
+
+				Method add = ev.Add;
+				if (add != null)
+				{
+					add.Modifiers |= TypeMemberModifiers.Final | TypeMemberModifiers.Virtual;
+				}
+
+				Method remove = ev.Remove;
+				if (remove != null)
+				{
+					remove.Modifiers |= TypeMemberModifiers.Final | TypeMemberModifiers.Virtual;
+				}
+
+				Method raise = ev.Remove;
+				if (raise != null)
+				{
+					raise.Modifiers |= TypeMemberModifiers.Final | TypeMemberModifiers.Virtual;
+				}
+
+				_context.TraceInfo("{0}: Event {1} implements {2}", ev.LexicalInfo, ev, entity);
+				return;
+			}
+
+			node.Members.Add(CodeBuilder.CreateAbstractEvent(baseTypeRef.LexicalInfo, entity));
+			AbstractMemberNotImplemented(node, baseTypeRef, entity);
+		}
+
 		void ResolveAbstractMethod(ClassDefinition node,
 			TypeReference baseTypeRef,
 			IMethod entity)
@@ -238,7 +304,7 @@ namespace Boo.Lang.Compiler.Steps
 					&& IsCorrectExplicitMemberImplOrNoExplicitMemberAtAll(member, entity))
 				{
 					Method method = (Method)member;
-					if (TypeSystemServices.CheckOverrideSignature((IMethod)GetEntity(method), entity))
+					if (TypeSystemServices.CheckOverrideSignature(GetEntity(method), entity))
 					{	
 						if (IsUnknown(method.ReturnType))
 						{
@@ -286,15 +352,32 @@ namespace Boo.Lang.Compiler.Steps
 		
 		void AbstractMemberNotImplemented(ClassDefinition node, TypeReference baseTypeRef, IMember member)
 		{
-			if (!node.IsAbstract)
+			if (IsValueType(node))
+			{
+				Error(CompilerErrorFactory.ValueTypeCantHaveAbstractMember(baseTypeRef, node.FullName, GetAbstractMemberSignature(member)));
+			}
+			else if (!node.IsAbstract)
 			{
 				Warnings.Add(
 					CompilerWarningFactory.AbstractMemberNotImplemented(baseTypeRef,
-					node.FullName, member.FullName));
+					node.FullName, GetAbstractMemberSignature(member)));
 				_newAbstractClasses.AddUnique(node);
 			}
 		}
-		
+
+		private bool IsValueType(ClassDefinition node)
+		{
+			return ((IType)node.Entity).IsValueType;
+		}
+
+		private string GetAbstractMemberSignature(IMember member)
+		{
+			IMethod method = member as IMethod;
+			return method != null
+				? TypeSystemServices.GetSignature(method)
+				: member.FullName;
+		}
+
 		void ResolveInterfaceMembers(ClassDefinition node,
 			TypeReference baseTypeRef,
 			IType baseType)
@@ -338,6 +421,17 @@ namespace Boo.Lang.Compiler.Steps
 						}
 						break;
 					}
+
+					case EntityType.Event:
+					{
+						IEvent ev = (IEvent)member;
+						if (ev.IsAbstract)
+						{
+							ResolveAbstractEvent(node, baseTypeRef, ev);
+						}
+						break;
+					}
+					
 				}
 			}
 		}
@@ -368,6 +462,12 @@ namespace Boo.Lang.Compiler.Steps
 					ResolveClassAbstractProperty(node, baseTypeRef, (IProperty)member);
 					break;
 				}
+
+				case EntityType.Event:
+				{
+					ResolveAbstractEvent(node, baseTypeRef, (IEvent)member);
+					break;
+				}
 				
 				default:
 				{
