@@ -26,16 +26,20 @@
 // THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #endregion
 
-using System;
-using System.Reflection;
-using System.Text;
-using Boo.Lang.Compiler;
-using Boo.Lang.Compiler.Ast;
-
 namespace Boo.Lang.Compiler.TypeSystem
 {
+	using System;
+	using System.Collections;
+	using System.Reflection;
+	using System.Text;
+	using Boo.Lang.Compiler.Ast;
+	using Attribute = Boo.Lang.Compiler.Ast.Attribute;
+	using Module = Boo.Lang.Compiler.Ast.Module;
+
 	public class TypeSystemServices
 	{
+		private static readonly bool Version1 = Environment.Version < new Version(2, 0);
+		
 		public DuckTypeImpl DuckType;
 		
 		public ExternalType IQuackFuType;
@@ -118,21 +122,21 @@ namespace Boo.Lang.Compiler.TypeSystem
 		
 		public ExternalType IDictionaryType;
 		
-		System.Collections.Hashtable _primitives = new System.Collections.Hashtable();
+		protected Hashtable _primitives = new Hashtable();
 		
-		System.Collections.Hashtable _entityCache = new System.Collections.Hashtable();
+		protected Hashtable _entityCache = new Hashtable();
 		
-		System.Collections.Hashtable _arrayCache = new System.Collections.Hashtable();
+		protected Hashtable _arrayCache = new Hashtable();
 		
-		System.Collections.Hashtable _anonymousCallableTypes = new System.Collections.Hashtable();
+		protected Hashtable _anonymousCallableTypes = new Hashtable();
 		
-		public static readonly IType ErrorEntity = Boo.Lang.Compiler.TypeSystem.Error.Default;
+		public static readonly IType ErrorEntity = Error.Default;
 		
 		public readonly BooCodeBuilder CodeBuilder;
 		
 		StringBuilder _buffer = new StringBuilder();
 		
-		Boo.Lang.Compiler.Ast.Module _anonymousTypesModule;
+		Module _anonymousTypesModule;
 		
 		CompilerContext _context;
 		
@@ -151,13 +155,13 @@ namespace Boo.Lang.Compiler.TypeSystem
 
 			CodeBuilder = new BooCodeBuilder(this);
 			
-			Cache(typeof(Boo.Lang.Builtins.duck), DuckType = new DuckTypeImpl(this));
-			Cache(IQuackFuType = new ExternalType(this, typeof(Boo.Lang.IQuackFu)));
+			Cache(typeof(Builtins.duck), DuckType = new DuckTypeImpl(this));
+			Cache(IQuackFuType = new ExternalType(this, typeof(IQuackFu)));
 			Cache(VoidType = new VoidTypeImpl(this));
 			Cache(ObjectType = new ExternalType(this, Types.Object));
 			Cache(RegexType = new ExternalType(this, Types.Regex));
-			Cache(ValueTypeType = new ExternalType(this, typeof(System.ValueType)));
-			Cache(EnumType = new ExternalType(this, typeof(System.Enum)));
+			Cache(ValueTypeType = new ExternalType(this, typeof(ValueType)));
+			Cache(EnumType = new ExternalType(this, typeof(Enum)));
 			Cache(ArrayType = new ExternalType(this, Types.Array));
 			Cache(TypeType = new ExternalType(this, Types.Type));
 			Cache(StringType = new ExternalType(this, Types.String));
@@ -182,7 +186,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			Cache(HashType = new ExternalType(this, Types.Hash));
 			Cache(ICallableType = new ExternalType(this, Types.ICallable));
 			Cache(IEnumerableType = new ExternalType(this, Types.IEnumerable));
-			Cache(IEnumeratorType = new ExternalType(this, typeof(System.Collections.IEnumerator)));
+			Cache(IEnumeratorType = new ExternalType(this, typeof(IEnumerator)));
 			Cache(ICollectionType = new ExternalType(this, Types.ICollection));
 			Cache(IListType = new ExternalType(this, Types.IList));
 			Cache(IDictionaryType = new ExternalType(this, Types.IDictionary));
@@ -368,6 +372,22 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			return (TypeType == type) || IsCallableType(type) || IsDuckType(type);
 		}
+
+		public bool IsDuckTyped(Expression expression)
+		{
+			IType type = expression.ExpressionType;
+			return null != type && this.IsDuckType(type);
+		}
+
+		public bool IsQuackBuiltin(Expression node)
+		{
+			return IsQuackBuiltin(GetOptionalEntity(node));
+		}
+		
+		public bool IsQuackBuiltin(IEntity entity)
+		{
+			return BuiltinFunction.Quack == entity;
+		}
 		
 		public bool IsDuckType(IType type)
 		{
@@ -375,19 +395,21 @@ namespace Boo.Lang.Compiler.TypeSystem
 			{
 				throw new ArgumentNullException("type");
 			}
-			return ((type == DuckType)  ||
-			        (_context.Parameters.Ducky &&
-			         (type == ObjectType || type.IsSubclassOf(IQuackFuType))));
+			return (
+				(type == DuckType)
+				|| (_context.Parameters.Ducky
+					&& (type == ObjectType
+						|| type.IsSubclassOf(IQuackFuType))));
 		}
 		
 		bool IsCallableType(IType type)
 		{
 			return (ICallableType.IsAssignableFrom(type)) ||
-				(type is Boo.Lang.Compiler.TypeSystem.ICallableType);
+				(type is ICallableType);
 		}
 		
 		public AnonymousCallableType GetCallableType(IMethod method)
-		{
+		{	
 			CallableSignature signature = new CallableSignature(method);
 			return GetCallableType(signature);
 		}
@@ -460,11 +482,11 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 		}
 		
-		public Boo.Lang.Compiler.Ast.Module GetAnonymousTypesModule()
+		public Module GetAnonymousTypesModule()
 		{
 			if (null == _anonymousTypesModule)
 			{
-				_anonymousTypesModule = new Boo.Lang.Compiler.Ast.Module();
+				_anonymousTypesModule = new Module();
 				_anonymousTypesModule.Entity = new ModuleEntity(_context.NameResolutionService,
 																this,
 																_anonymousTypesModule);
@@ -566,6 +588,25 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return false;
 		}
 		
+		public static string GetReferenceTypeName(Type t)
+		{
+			string name = t.FullName;
+			if (!(name.EndsWith("&")))
+			{
+				return name+"&";
+			}
+			return name;		}
+		
+		public static string DeReferenceTypeName(IType t)
+		{
+			string name = t.FullName;
+			if (name.EndsWith("&"))
+			{
+				return name.Substring(0, t.FullName.Length-1);
+			}
+			return name;
+		}
+		
 		public static bool CheckOverrideSignature(IMethod impl, IMethod baseMethod)
 		{
 			IParameter[] implParameters = impl.GetParameters();
@@ -579,7 +620,21 @@ namespace Boo.Lang.Compiler.TypeSystem
 			{
 				for (int i=0; i<implParameters.Length; ++i)
 				{
-					if (implParameters[i].Type != baseParameters[i].Type)
+					IType impltype = implParameters[i].Type;
+					IType basetype = baseParameters[i].Type;
+					
+					if (basetype.IsByRef)
+					{
+						if (!(implParameters[i].IsByRef))
+						{
+							return false;
+						}
+						if (DeReferenceTypeName(basetype) != impltype.FullName)
+						{
+							return false;
+						}
+					}
+					else if (impltype != basetype)
 					{
 						return false;
 					}
@@ -602,7 +657,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return type.IsEnum || type == this.CharType;
 		}
 		
-		public static bool ContainsMethodsOnly(Boo.Lang.List members)
+		public static bool ContainsMethodsOnly(List members)
 		{
 			foreach (IEntity member in members)
 			{
@@ -718,12 +773,12 @@ namespace Boo.Lang.Compiler.TypeSystem
 		
 		public static IEntity[] GetAllMembers(INamespace entity)
 		{
-			Boo.Lang.List members = new Boo.Lang.List();
+			List members = new List();
 			GetAllMembers(members, entity);
 			return (IEntity[])members.ToArray(typeof(IEntity));
 		}
 		
-		private static void GetAllMembers(Boo.Lang.List members, INamespace entity)
+		private static void GetAllMembers(List members, INamespace entity)
 		{
 			if (null == entity)
 			{
@@ -742,18 +797,24 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 		}
 		
+		static object EntityAnnotationKey = new object();
+		
+		public static void Bind(Node node, IEntity entity)
+		{
+			if (null == node) throw new ArgumentNullException("node");
+			node[EntityAnnotationKey] = entity;
+		}
+		
+		public static IEntity GetOptionalEntity(Node node)
+		{
+			if (null == node) throw new ArgumentNullException("node");
+			return (IEntity)node[EntityAnnotationKey];
+		}
+		
 		public static IEntity GetEntity(Node node)
 		{
-			if (null == node)
-			{
-				throw new ArgumentNullException("node");
-			}
-			
-			IEntity tag = node.Entity;
-			if (null == tag)
-			{
-				InvalidNode(node);
-			}
+			IEntity tag = GetOptionalEntity(node);
+			if (null == tag) InvalidNode(node);
 			return tag;
 		}
 		
@@ -779,7 +840,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return ((ITypedEntity)GetEntity(node)).Type;
 		}
 		
-		public IType Map(System.Type type)
+		public IType Map(Type type)
 		{
 			ExternalType entity = (ExternalType)_entityCache[type];
 			if (null == entity)
@@ -838,7 +899,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 		}
 			
-		public IParameter[] Map(Boo.Lang.Compiler.Ast.ParameterDeclarationCollection parameters)
+		public IParameter[] Map(ParameterDeclarationCollection parameters)
 		{
 			IParameter[] mapped = new IParameter[parameters.Count];
 			for (int i=0; i<mapped.Length; ++i)
@@ -848,7 +909,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return mapped;
 		}
 		
-		public IParameter[] Map(System.Reflection.ParameterInfo[] parameters)
+		public IParameter[] Map(ParameterInfo[] parameters)
 		{
 			IParameter[] mapped = new IParameter[parameters.Length];
 			for (int i=0; i<parameters.Length; ++i)
@@ -882,7 +943,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return entity;
 		}
 		
-		public IEntity Map(System.Reflection.MemberInfo[] info)
+		public IEntity Map(MemberInfo[] info)
 		{
 			if (info.Length > 1)
 			{
@@ -900,7 +961,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return null;
 		}
 		
-		public IEntity Map(System.Reflection.MemberInfo mi)
+		public IEntity Map(MemberInfo mi)
 		{
 			IEntity tag = (IEntity)_entityCache[GetCacheKey(mi)];
 			if (null == tag)
@@ -909,35 +970,35 @@ namespace Boo.Lang.Compiler.TypeSystem
 				{
 					case MemberTypes.Method:
 					{
-						return Map((System.Reflection.MethodInfo)mi);
+						return Map((MethodInfo)mi);
 					}
 					
 					case MemberTypes.Constructor:
 					{
-						return Map((System.Reflection.ConstructorInfo)mi);
+						return Map((ConstructorInfo)mi);
 					}
 					
 					case MemberTypes.Field:
 					{
-						tag = new ExternalField(this, (System.Reflection.FieldInfo)mi);
+						tag = new ExternalField(this, (FieldInfo)mi);
 						break;
 					}
 					
 					case MemberTypes.Property:
 					{
-						tag = new ExternalProperty(this, (System.Reflection.PropertyInfo)mi);
+						tag = new ExternalProperty(this, (PropertyInfo)mi);
 						break;
 					}
 					
 					case MemberTypes.Event:
 					{
-						tag = new ExternalEvent(this, (System.Reflection.EventInfo)mi);
+						tag = new ExternalEvent(this, (EventInfo)mi);
 						break;
 					}
 					
 					case MemberTypes.NestedType:
 					{
-						return Map((System.Type)mi);
+						return Map((Type)mi);
 					}
 					
 					default:
@@ -950,12 +1011,12 @@ namespace Boo.Lang.Compiler.TypeSystem
 			return tag;
 		}
 		
-		public string GetSignature(IMethod method)
+		public string GetSignature(IEntityWithParameters method)
 		{
 			return GetSignature(method, true);
 		}
 		
-		public string GetSignature(IMethod method, bool includeFullName)
+		public string GetSignature(IEntityWithParameters method, bool includeFullName)
 		{
 			_buffer.Length = 0;
 			if (includeFullName)
@@ -972,13 +1033,14 @@ namespace Boo.Lang.Compiler.TypeSystem
 			for (int i=0; i<parameters.Length; ++i)
 			{
 				if (i > 0) { _buffer.Append(", "); }
+				if (method.AcceptVarArgs && i == parameters.Length-1) { _buffer.Append('*'); }
 				_buffer.Append(parameters[i].Type.FullName);
 			}
 			_buffer.Append(")");
 			return _buffer.ToString();
 		}
 		
-		public object GetCacheKey(System.Reflection.MemberInfo mi)
+		public object GetCacheKey(MemberInfo mi)
 		{
 			return mi;
 		}
@@ -1036,12 +1098,12 @@ namespace Boo.Lang.Compiler.TypeSystem
 			AddBuiltin(BuiltinFunction.Switch);
 		}
 		
-		void AddPrimitiveType(string name, ExternalType type)
+		protected void AddPrimitiveType(string name, ExternalType type)
 		{
 			_primitives[name] = type;
 		}
 		
-		void AddBuiltin(BuiltinFunction function)
+		protected void AddBuiltin(BuiltinFunction function)
 		{
 			_primitives[function.Name] = function;
 		}
@@ -1059,7 +1121,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		Method CreateBeginInvokeMethod(ICallableType anonymousType)
 		{
 			Method method = CodeBuilder.CreateRuntimeMethod("BeginInvoke", Map(typeof(IAsyncResult)),
-												anonymousType.GetSignature().Parameters);
+			                                                anonymousType.GetSignature().Parameters, false);
 												
 			int delta=method.Parameters.Count;
 			method.Parameters.Add(
@@ -1074,7 +1136,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			InternalMethod beginInvokeEntity = (InternalMethod)beginInvoke.Entity;
 			
 			Method overload = CodeBuilder.CreateMethod("BeginInvoke", Map(typeof(IAsyncResult)),
-										TypeMemberModifiers.Public);
+										TypeMemberModifiers.Public|TypeMemberModifiers.Virtual);
 			CodeBuilder.DeclareParameters(overload, 1, anonymousType.GetSignature().Parameters);
 			
 			mie = CodeBuilder.CreateMethodInvocation(
@@ -1120,7 +1182,20 @@ namespace Boo.Lang.Compiler.TypeSystem
 		{
 			CallableSignature signature = anonymousType.GetSignature();
 			Method method = CodeBuilder.CreateRuntimeMethod("EndInvoke", signature.ReturnType);
-			int delta=method.Parameters.Count;
+			
+			int delta=1;
+			foreach(IParameter p in signature.Parameters)
+			{
+				if (p.IsByRef)
+				{
+					method.Parameters.Add(
+						CodeBuilder.CreateParameterDeclaration(++delta,
+								p.Name,
+								p.Type,
+								true));
+				}			
+			}
+			delta=method.Parameters.Count;
 			method.Parameters.Add(
 				CodeBuilder.CreateParameterDeclaration(delta+1, "result", Map(typeof(IAsyncResult))));
 			return method;
@@ -1129,7 +1204,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		Method CreateInvokeMethod(AnonymousCallableType anonymousType)
 		{
 			CallableSignature signature = anonymousType.GetSignature();
-			return CodeBuilder.CreateRuntimeMethod("Invoke", signature.ReturnType, signature.Parameters);
+			return CodeBuilder.CreateRuntimeMethod("Invoke", signature.ReturnType, signature.Parameters, signature.AcceptVarArgs);
 		}
 		
 		public IConstructor GetDefaultConstructor(IType type)
@@ -1166,7 +1241,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			}
 			
 			IType enumeratorItemTypeAttribute = Map(typeof(EnumeratorItemTypeAttribute));
-			foreach (Boo.Lang.Compiler.Ast.Attribute attribute in internalType.TypeDefinition.Attributes)
+			foreach (Attribute attribute in internalType.TypeDefinition.Attributes)
 			{
 				IConstructor constructor = GetEntity(attribute) as IConstructor;
 				if (null != constructor)
@@ -1191,7 +1266,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 		
 		protected virtual IType CreateConcreteCallableType(Node sourceNode, AnonymousCallableType anonymousType)
 		{
-			Boo.Lang.Compiler.Ast.Module module = GetAnonymousTypesModule();
+			Module module = GetAnonymousTypesModule();
 			
 			string name = string.Format("___callable{0}", module.Members.Count);
 			ClassDefinition cd = CreateCallableDefinition(name);
@@ -1202,8 +1277,13 @@ namespace Boo.Lang.Compiler.TypeSystem
 			
 			Method beginInvoke = CreateBeginInvokeMethod(anonymousType);
 			cd.Members.Add(beginInvoke);
-			cd.Members.Add(CreateBeginInvokeCallbackOnlyOverload(anonymousType, beginInvoke));
-			cd.Members.Add(CreateBeginInvokeSimplerOverload(anonymousType, beginInvoke));
+			
+			// XXX: find an alternative way to support BeginInvoke overloads... 
+			if (Version1)
+			{
+				cd.Members.Add(CreateBeginInvokeCallbackOnlyOverload(anonymousType, beginInvoke));
+				cd.Members.Add(CreateBeginInvokeSimplerOverload(anonymousType, beginInvoke));
+			}
 			
 			cd.Members.Add(CreateEndInvokeMethod(anonymousType));
 			_anonymousTypesModule.Members.Add(cd);
@@ -1231,7 +1311,7 @@ namespace Boo.Lang.Compiler.TypeSystem
 			{
 			}
 			
-			override public bool Resolve(Boo.Lang.List targetList, string name, EntityType flags)
+			override public bool Resolve(List targetList, string name, EntityType flags)
 			{
 				return false;
 			}
