@@ -1,4 +1,4 @@
-﻿#region license
+#region license
 // Copyright (c) 2003, 2004, 2005 Rodrigo B. de Oliveira (rbo@acm.org)
 // All rights reserved.
 // 
@@ -59,7 +59,7 @@ namespace Boo.Lang.Parser
 	using LexerSharedInputState           = antlr.LexerSharedInputState;
 	using BitSet                          = antlr.collections.impl.BitSet;
 	
-	internal 	class BooExpressionLexer : antlr.CharScanner	, TokenStream
+	public 	class BooExpressionLexer : antlr.CharScanner	, TokenStream
 	 {
 		public const int EOF = 1;
 		public const int NULL_TREE_LOOKAHEAD = 3;
@@ -71,122 +71,126 @@ namespace Boo.Lang.Parser
 		public const int ABSTRACT = 9;
 		public const int AND = 10;
 		public const int AS = 11;
-		public const int BREAK = 12;
-		public const int CONTINUE = 13;
-		public const int CALLABLE = 14;
-		public const int CAST = 15;
-		public const int CHAR = 16;
-		public const int CLASS = 17;
-		public const int CONSTRUCTOR = 18;
-		public const int DEF = 19;
-		public const int DESTRUCTOR = 20;
-		public const int DO = 21;
-		public const int ELIF = 22;
-		public const int ELSE = 23;
-		public const int ENSURE = 24;
-		public const int ENUM = 25;
-		public const int EVENT = 26;
-		public const int EXCEPT = 27;
-		public const int FAILURE = 28;
-		public const int FINAL = 29;
-		public const int FROM = 30;
-		public const int FOR = 31;
-		public const int FALSE = 32;
-		public const int GET = 33;
-		public const int GIVEN = 34;
-		public const int GOTO = 35;
-		public const int IMPORT = 36;
-		public const int INTERFACE = 37;
-		public const int INTERNAL = 38;
-		public const int IS = 39;
-		public const int ISA = 40;
-		public const int IF = 41;
-		public const int IN = 42;
-		public const int NOT = 43;
-		public const int NULL = 44;
-		public const int OR = 45;
-		public const int OTHERWISE = 46;
-		public const int OVERRIDE = 47;
-		public const int PASS = 48;
-		public const int NAMESPACE = 49;
-		public const int PUBLIC = 50;
-		public const int PROTECTED = 51;
-		public const int PRIVATE = 52;
-		public const int RAISE = 53;
-		public const int RETURN = 54;
-		public const int RETRY = 55;
-		public const int SET = 56;
-		public const int SELF = 57;
-		public const int SUPER = 58;
-		public const int STATIC = 59;
-		public const int SUCCESS = 60;
-		public const int STRUCT = 61;
-		public const int TRY = 62;
-		public const int TRANSIENT = 63;
-		public const int TRUE = 64;
-		public const int TYPEOF = 65;
-		public const int UNLESS = 66;
-		public const int VIRTUAL = 67;
-		public const int WHEN = 68;
-		public const int WHILE = 69;
-		public const int YIELD = 70;
-		public const int EOS = 71;
-		public const int TRIPLE_QUOTED_STRING = 72;
-		public const int DOUBLE_QUOTED_STRING = 73;
-		public const int SINGLE_QUOTED_STRING = 74;
-		public const int ID = 75;
-		public const int LPAREN = 76;
-		public const int RPAREN = 77;
-		public const int ASSIGN = 78;
-		public const int SUBTRACT = 79;
-		public const int LBRACK = 80;
-		public const int COMMA = 81;
-		public const int RBRACK = 82;
-		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 83;
-		public const int DOT = 84;
-		public const int COLON = 85;
-		public const int MULTIPLY = 86;
-		public const int BITWISE_OR = 87;
-		public const int LBRACE = 88;
-		public const int RBRACE = 89;
-		public const int INPLACE_BITWISE_OR = 90;
-		public const int INPLACE_BITWISE_AND = 91;
-		public const int INPLACE_SHIFT_LEFT = 92;
-		public const int INPLACE_SHIFT_RIGHT = 93;
-		public const int CMP_OPERATOR = 94;
-		public const int GREATER_THAN = 95;
-		public const int LESS_THAN = 96;
-		public const int ADD = 97;
-		public const int EXCLUSIVE_OR = 98;
-		public const int DIVISION = 99;
-		public const int MODULUS = 100;
-		public const int BITWISE_AND = 101;
-		public const int SHIFT_LEFT = 102;
-		public const int SHIFT_RIGHT = 103;
-		public const int EXPONENTIATION = 104;
-		public const int INCREMENT = 105;
-		public const int DECREMENT = 106;
-		public const int INT = 107;
-		public const int LONG = 108;
-		public const int RE_LITERAL = 109;
-		public const int DOUBLE = 110;
-		public const int TIMESPAN = 111;
-		public const int LINE_CONTINUATION = 112;
-		public const int SL_COMMENT = 113;
-		public const int ML_COMMENT = 114;
-		public const int WS = 115;
-		public const int X_RE_LITERAL = 116;
-		public const int NEWLINE = 117;
-		public const int ESCAPED_EXPRESSION = 118;
-		public const int DQS_ESC = 119;
-		public const int SQS_ESC = 120;
-		public const int SESC = 121;
-		public const int RE_CHAR = 122;
-		public const int X_RE_CHAR = 123;
-		public const int RE_ESC = 124;
-		public const int ID_LETTER = 125;
-		public const int DIGIT = 126;
-		public const int HEXDIGIT = 127;
+		public const int AST = 12;
+		public const int BREAK = 13;
+		public const int CONTINUE = 14;
+		public const int CALLABLE = 15;
+		public const int CAST = 16;
+		public const int CHAR = 17;
+		public const int CLASS = 18;
+		public const int CONSTRUCTOR = 19;
+		public const int DEF = 20;
+		public const int DESTRUCTOR = 21;
+		public const int DO = 22;
+		public const int ELIF = 23;
+		public const int ELSE = 24;
+		public const int ENSURE = 25;
+		public const int ENUM = 26;
+		public const int EVENT = 27;
+		public const int EXCEPT = 28;
+		public const int FAILURE = 29;
+		public const int FINAL = 30;
+		public const int FROM = 31;
+		public const int FOR = 32;
+		public const int FALSE = 33;
+		public const int GET = 34;
+		public const int GIVEN = 35;
+		public const int GOTO = 36;
+		public const int IMPORT = 37;
+		public const int INTERFACE = 38;
+		public const int INTERNAL = 39;
+		public const int IS = 40;
+		public const int ISA = 41;
+		public const int IF = 42;
+		public const int IN = 43;
+		public const int NOT = 44;
+		public const int NULL = 45;
+		public const int OR = 46;
+		public const int OTHERWISE = 47;
+		public const int OVERRIDE = 48;
+		public const int PASS = 49;
+		public const int NAMESPACE = 50;
+		public const int PUBLIC = 51;
+		public const int PROTECTED = 52;
+		public const int PRIVATE = 53;
+		public const int RAISE = 54;
+		public const int REF = 55;
+		public const int RETURN = 56;
+		public const int RETRY = 57;
+		public const int SET = 58;
+		public const int SELF = 59;
+		public const int SUPER = 60;
+		public const int STATIC = 61;
+		public const int SUCCESS = 62;
+		public const int STRUCT = 63;
+		public const int TRY = 64;
+		public const int TRANSIENT = 65;
+		public const int TRUE = 66;
+		public const int TYPEOF = 67;
+		public const int UNLESS = 68;
+		public const int VIRTUAL = 69;
+		public const int WHEN = 70;
+		public const int WHILE = 71;
+		public const int YIELD = 72;
+		public const int EOS = 73;
+		public const int TRIPLE_QUOTED_STRING = 74;
+		public const int DOUBLE_QUOTED_STRING = 75;
+		public const int SINGLE_QUOTED_STRING = 76;
+		public const int ID = 77;
+		public const int LPAREN = 78;
+		public const int RPAREN = 79;
+		public const int ASSIGN = 80;
+		public const int SUBTRACT = 81;
+		public const int LBRACK = 82;
+		public const int COMMA = 83;
+		public const int RBRACK = 84;
+		public const int ASSEMBLY_ATTRIBUTE_BEGIN = 85;
+		public const int DOT = 86;
+		public const int COLON = 87;
+		public const int MULTIPLY = 88;
+		public const int BITWISE_OR = 89;
+		public const int LBRACE = 90;
+		public const int RBRACE = 91;
+		public const int INPLACE_BITWISE_OR = 92;
+		public const int INPLACE_BITWISE_AND = 93;
+		public const int INPLACE_SHIFT_LEFT = 94;
+		public const int INPLACE_SHIFT_RIGHT = 95;
+		public const int CMP_OPERATOR = 96;
+		public const int GREATER_THAN = 97;
+		public const int LESS_THAN = 98;
+		public const int ADD = 99;
+		public const int EXCLUSIVE_OR = 100;
+		public const int DIVISION = 101;
+		public const int MODULUS = 102;
+		public const int BITWISE_AND = 103;
+		public const int SHIFT_LEFT = 104;
+		public const int SHIFT_RIGHT = 105;
+		public const int EXPONENTIATION = 106;
+		public const int INCREMENT = 107;
+		public const int DECREMENT = 108;
+		public const int ONES_COMPLEMENT = 109;
+		public const int INT = 110;
+		public const int LONG = 111;
+		public const int RE_LITERAL = 112;
+		public const int DOUBLE = 113;
+		public const int FLOAT = 114;
+		public const int TIMESPAN = 115;
+		public const int LINE_CONTINUATION = 116;
+		public const int SL_COMMENT = 117;
+		public const int ML_COMMENT = 118;
+		public const int WS = 119;
+		public const int X_RE_LITERAL = 120;
+		public const int NEWLINE = 121;
+		public const int ESCAPED_EXPRESSION = 122;
+		public const int DQS_ESC = 123;
+		public const int SQS_ESC = 124;
+		public const int SESC = 125;
+		public const int RE_CHAR = 126;
+		public const int X_RE_CHAR = 127;
+		public const int RE_ESC = 128;
+		public const int ID_LETTER = 129;
+		public const int DIGIT = 130;
+		public const int HEXDIGIT = 131;
 		
 		
 	
@@ -220,68 +224,70 @@ namespace Boo.Lang.Parser
 			caseSensitiveLiterals = true;
 			setCaseSensitive(true);
 			literals = new Hashtable(100, (float) 0.4, null, Comparer.Default);
-			literals.Add("public", 50);
-			literals.Add("namespace", 49);
-			literals.Add("break", 12);
-			literals.Add("while", 69);
-			literals.Add("otherwise", 46);
-			literals.Add("raise", 53);
-			literals.Add("typeof", 65);
+			literals.Add("public", 51);
+			literals.Add("namespace", 50);
+			literals.Add("break", 13);
+			literals.Add("while", 71);
+			literals.Add("otherwise", 47);
+			literals.Add("raise", 54);
+			literals.Add("typeof", 67);
 			literals.Add("and", 10);
-			literals.Add("failure", 28);
-			literals.Add("not", 43);
-			literals.Add("return", 54);
-			literals.Add("pass", 48);
-			literals.Add("from", 30);
-			literals.Add("null", 44);
-			literals.Add("def", 19);
-			literals.Add("given", 34);
-			literals.Add("protected", 51);
-			literals.Add("retry", 55);
-			literals.Add("when", 68);
-			literals.Add("class", 17);
-			literals.Add("do", 21);
-			literals.Add("except", 27);
-			literals.Add("event", 26);
-			literals.Add("unless", 66);
-			literals.Add("super", 58);
-			literals.Add("set", 56);
-			literals.Add("transient", 63);
-			literals.Add("constructor", 18);
-			literals.Add("interface", 37);
-			literals.Add("is", 39);
-			literals.Add("internal", 38);
-			literals.Add("final", 29);
-			literals.Add("yield", 70);
-			literals.Add("or", 45);
-			literals.Add("destructor", 20);
-			literals.Add("if", 41);
-			literals.Add("success", 60);
-			literals.Add("override", 47);
+			literals.Add("failure", 29);
+			literals.Add("not", 44);
+			literals.Add("return", 56);
+			literals.Add("pass", 49);
+			literals.Add("from", 31);
+			literals.Add("null", 45);
+			literals.Add("def", 20);
+			literals.Add("given", 35);
+			literals.Add("protected", 52);
+			literals.Add("ref", 55);
+			literals.Add("retry", 57);
+			literals.Add("when", 70);
+			literals.Add("class", 18);
+			literals.Add("do", 22);
+			literals.Add("except", 28);
+			literals.Add("event", 27);
+			literals.Add("ast", 12);
+			literals.Add("unless", 68);
+			literals.Add("super", 60);
+			literals.Add("set", 58);
+			literals.Add("transient", 65);
+			literals.Add("constructor", 19);
+			literals.Add("interface", 38);
+			literals.Add("is", 40);
+			literals.Add("internal", 39);
+			literals.Add("final", 30);
+			literals.Add("yield", 72);
+			literals.Add("or", 46);
+			literals.Add("destructor", 21);
+			literals.Add("if", 42);
+			literals.Add("success", 62);
+			literals.Add("override", 48);
 			literals.Add("as", 11);
-			literals.Add("try", 62);
-			literals.Add("goto", 35);
-			literals.Add("enum", 25);
-			literals.Add("isa", 40);
-			literals.Add("for", 31);
-			literals.Add("char", 16);
-			literals.Add("private", 52);
-			literals.Add("false", 32);
-			literals.Add("static", 59);
+			literals.Add("try", 64);
+			literals.Add("goto", 36);
+			literals.Add("enum", 26);
+			literals.Add("isa", 41);
+			literals.Add("for", 32);
+			literals.Add("char", 17);
+			literals.Add("private", 53);
+			literals.Add("false", 33);
+			literals.Add("static", 61);
 			literals.Add("abstract", 9);
-			literals.Add("callable", 14);
-			literals.Add("get", 33);
-			literals.Add("continue", 13);
-			literals.Add("cast", 15);
-			literals.Add("struct", 61);
-			literals.Add("else", 23);
-			literals.Add("import", 36);
-			literals.Add("elif", 22);
-			literals.Add("in", 42);
-			literals.Add("self", 57);
-			literals.Add("ensure", 24);
-			literals.Add("true", 64);
-			literals.Add("virtual", 67);
+			literals.Add("callable", 15);
+			literals.Add("get", 34);
+			literals.Add("continue", 14);
+			literals.Add("cast", 16);
+			literals.Add("struct", 63);
+			literals.Add("else", 24);
+			literals.Add("import", 37);
+			literals.Add("elif", 23);
+			literals.Add("in", 43);
+			literals.Add("self", 59);
+			literals.Add("ensure", 25);
+			literals.Add("true", 66);
+			literals.Add("virtual", 69);
 		}
 		
 		override public IToken nextToken()			//throws TokenStreamException
@@ -416,6 +422,12 @@ tryAgain:
 							theRetToken = returnToken_;
 							break;
 						}
+						case '~':
+						{
+							mONES_COMPLEMENT(true);
+							theRetToken = returnToken_;
+							break;
+						}
 						case '=':
 						{
 							mASSIGN(true);
@@ -700,7 +712,78 @@ _loop7_breakloop:					;
 _loop11_breakloop:				;
 			}    // ( ... )+
 			{
-				if ((cached_LA1=='L'||cached_LA1=='l'))
+				if ((cached_LA1=='E'||cached_LA1=='e'))
+				{
+					{
+						switch ( cached_LA1 )
+						{
+						case 'e':
+						{
+							match('e');
+							break;
+						}
+						case 'E':
+						{
+							match('E');
+							break;
+						}
+						default:
+						{
+							throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+						}
+						 }
+					}
+					{
+						switch ( cached_LA1 )
+						{
+						case '+':
+						{
+							match('+');
+							break;
+						}
+						case '-':
+						{
+							match('-');
+							break;
+						}
+						case '0':  case '1':  case '2':  case '3':
+						case '4':  case '5':  case '6':  case '7':
+						case '8':  case '9':
+						{
+							break;
+						}
+						default:
+						{
+							throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+						}
+						 }
+					}
+					{ // ( ... )+
+						int _cnt16=0;
+						for (;;)
+						{
+							if (((cached_LA1 >= '0' && cached_LA1 <= '9')))
+							{
+								mDIGIT(false);
+							}
+							else
+							{
+								if (_cnt16 >= 1) { goto _loop16_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							}
+							
+							_cnt16++;
+						}
+_loop16_breakloop:						;
+					}    // ( ... )+
+				}
+				else {
+				}
+				
+			}
+			{
+				switch ( cached_LA1 )
+				{
+				case 'L':  case 'l':
 				{
 					{
 						switch ( cached_LA1 )
@@ -725,88 +808,221 @@ _loop11_breakloop:				;
 					{
 						_ttype = LONG;
 					}
+					break;
 				}
-				else {
+				case 'F':  case 'f':
+				{
 					{
 						{
-							if (((cached_LA1=='.'))&&(BooLexer.IsDigit(LA(2))))
+							switch ( cached_LA1 )
 							{
-								{
-									match('.');
-									{ // ( ... )+
-										int _cnt18=0;
-										for (;;)
-										{
-											if (((cached_LA1 >= '0' && cached_LA1 <= '9')))
-											{
-												mDIGIT(false);
-											}
-											else
-											{
-												if (_cnt18 >= 1) { goto _loop18_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
-											}
-											
-											_cnt18++;
-										}
-_loop18_breakloop:										;
-									}    // ( ... )+
-								}
-								if (0==inputState.guessing)
-								{
-									_ttype = DOUBLE;
-								}
+							case 'f':
+							{
+								match('f');
+								break;
 							}
-							else {
+							case 'F':
+							{
+								match('F');
+								break;
 							}
-							
+							default:
+							{
+								throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+							}
+							 }
 						}
+						if (0==inputState.guessing)
 						{
-							if ((cached_LA1=='d'||cached_LA1=='h'||cached_LA1=='m'||cached_LA1=='s'))
-							{
-								{
-									switch ( cached_LA1 )
-									{
-									case 's':
-									{
-										match('s');
-										break;
-									}
-									case 'h':
-									{
-										match('h');
-										break;
-									}
-									case 'd':
-									{
-										match('d');
-										break;
-									}
-									default:
-										if ((cached_LA1=='m') && (cached_LA2=='s'))
-										{
-											match("ms");
-										}
-										else if ((cached_LA1=='m') && (true)) {
-											match('m');
-										}
-									else
-									{
-										throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
-									}
-									break; }
-								}
-								if (0==inputState.guessing)
-								{
-									_ttype = TIMESPAN;
-								}
-							}
-							else {
-							}
-							
+							_ttype = FLOAT;
 						}
 					}
+					break;
 				}
-				
+				default:
+					{
+						{
+							{
+								if (((cached_LA1=='.'))&&(BooLexer.IsDigit(LA(2))))
+								{
+									{
+										match('.');
+										{ // ( ... )+
+											int _cnt25=0;
+											for (;;)
+											{
+												if (((cached_LA1 >= '0' && cached_LA1 <= '9')))
+												{
+													mDIGIT(false);
+												}
+												else
+												{
+													if (_cnt25 >= 1) { goto _loop25_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+												}
+												
+												_cnt25++;
+											}
+_loop25_breakloop:											;
+										}    // ( ... )+
+										{
+											if ((cached_LA1=='E'||cached_LA1=='e'))
+											{
+												{
+													switch ( cached_LA1 )
+													{
+													case 'e':
+													{
+														match('e');
+														break;
+													}
+													case 'E':
+													{
+														match('E');
+														break;
+													}
+													default:
+													{
+														throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+													}
+													 }
+												}
+												{
+													switch ( cached_LA1 )
+													{
+													case '+':
+													{
+														match('+');
+														break;
+													}
+													case '-':
+													{
+														match('-');
+														break;
+													}
+													case '0':  case '1':  case '2':  case '3':
+													case '4':  case '5':  case '6':  case '7':
+													case '8':  case '9':
+													{
+														break;
+													}
+													default:
+													{
+														throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+													}
+													 }
+												}
+												{ // ( ... )+
+													int _cnt30=0;
+													for (;;)
+													{
+														if (((cached_LA1 >= '0' && cached_LA1 <= '9')))
+														{
+															mDIGIT(false);
+														}
+														else
+														{
+															if (_cnt30 >= 1) { goto _loop30_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+														}
+														
+														_cnt30++;
+													}
+_loop30_breakloop:													;
+												}    // ( ... )+
+											}
+											else {
+											}
+											
+										}
+									}
+									{
+										if ((cached_LA1=='F'||cached_LA1=='f'))
+										{
+											{
+												{
+													switch ( cached_LA1 )
+													{
+													case 'f':
+													{
+														match('f');
+														break;
+													}
+													case 'F':
+													{
+														match('F');
+														break;
+													}
+													default:
+													{
+														throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+													}
+													 }
+												}
+												if (0==inputState.guessing)
+												{
+													_ttype = FLOAT;
+												}
+											}
+										}
+										else {
+											if (0==inputState.guessing)
+											{
+												_ttype = DOUBLE;
+											}
+										}
+										
+									}
+								}
+								else {
+								}
+								
+							}
+							{
+								if ((cached_LA1=='d'||cached_LA1=='h'||cached_LA1=='m'||cached_LA1=='s'))
+								{
+									{
+										switch ( cached_LA1 )
+										{
+										case 's':
+										{
+											match('s');
+											break;
+										}
+										case 'h':
+										{
+											match('h');
+											break;
+										}
+										case 'd':
+										{
+											match('d');
+											break;
+										}
+										default:
+											if ((cached_LA1=='m') && (cached_LA2=='s'))
+											{
+												match("ms");
+											}
+											else if ((cached_LA1=='m') && (true)) {
+												match('m');
+											}
+										else
+										{
+											throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+										}
+										break; }
+									}
+									if (0==inputState.guessing)
+									{
+										_ttype = TIMESPAN;
+									}
+								}
+								else {
+								}
+								
+							}
+						}
+					}
+				break; }
 			}
 		}
 		else
@@ -873,7 +1089,7 @@ _loop18_breakloop:										;
 			if (((cached_LA1 >= '0' && cached_LA1 <= '9')))
 			{
 				{ // ( ... )+
-					int _cnt24=0;
+					int _cnt39=0;
 					for (;;)
 					{
 						if (((cached_LA1 >= '0' && cached_LA1 <= '9')))
@@ -882,16 +1098,164 @@ _loop18_breakloop:										;
 						}
 						else
 						{
-							if (_cnt24 >= 1) { goto _loop24_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt39 >= 1) { goto _loop39_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt24++;
+						_cnt39++;
 					}
-_loop24_breakloop:					;
+_loop39_breakloop:					;
 				}    // ( ... )+
-				if (0==inputState.guessing)
 				{
-					_ttype = DOUBLE;
+					if ((cached_LA1=='E'||cached_LA1=='e'))
+					{
+						{
+							switch ( cached_LA1 )
+							{
+							case 'e':
+							{
+								match('e');
+								break;
+							}
+							case 'E':
+							{
+								match('E');
+								break;
+							}
+							default:
+							{
+								throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+							}
+							 }
+						}
+						{
+							switch ( cached_LA1 )
+							{
+							case '+':
+							{
+								match('+');
+								break;
+							}
+							case '-':
+							{
+								match('-');
+								break;
+							}
+							case '0':  case '1':  case '2':  case '3':
+							case '4':  case '5':  case '6':  case '7':
+							case '8':  case '9':
+							{
+								break;
+							}
+							default:
+							{
+								throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+							}
+							 }
+						}
+						{ // ( ... )+
+							int _cnt44=0;
+							for (;;)
+							{
+								if (((cached_LA1 >= '0' && cached_LA1 <= '9')))
+								{
+									mDIGIT(false);
+								}
+								else
+								{
+									if (_cnt44 >= 1) { goto _loop44_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+								}
+								
+								_cnt44++;
+							}
+_loop44_breakloop:							;
+						}    // ( ... )+
+					}
+					else {
+					}
+					
+				}
+				{
+					switch ( cached_LA1 )
+					{
+					case 'F':  case 'f':
+					{
+						{
+							{
+								switch ( cached_LA1 )
+								{
+								case 'f':
+								{
+									match('f');
+									break;
+								}
+								case 'F':
+								{
+									match('F');
+									break;
+								}
+								default:
+								{
+									throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+								}
+								 }
+							}
+							if (0==inputState.guessing)
+							{
+								_ttype = FLOAT;
+							}
+						}
+						break;
+					}
+					case 'd':  case 'h':  case 'm':  case 's':
+					{
+						{
+							{
+								switch ( cached_LA1 )
+								{
+								case 's':
+								{
+									match('s');
+									break;
+								}
+								case 'h':
+								{
+									match('h');
+									break;
+								}
+								case 'd':
+								{
+									match('d');
+									break;
+								}
+								default:
+									if ((cached_LA1=='m') && (cached_LA2=='s'))
+									{
+										match("ms");
+									}
+									else if ((cached_LA1=='m') && (true)) {
+										match('m');
+									}
+								else
+								{
+									throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
+								}
+								break; }
+							}
+							if (0==inputState.guessing)
+							{
+								_ttype = TIMESPAN;
+							}
+						}
+						break;
+					}
+					default:
+						{
+							if (0==inputState.guessing)
+							{
+								_ttype = DOUBLE;
+							}
+						}
+					break; }
 				}
 			}
 			else {
@@ -1218,11 +1582,11 @@ _loop24_breakloop:					;
 		int _ttype; IToken _token=null; int _begin=text.Length;
 		_ttype = DIVISION;
 		
-		bool synPredMatched50 = false;
+		bool synPredMatched75 = false;
 		if (((cached_LA1=='/') && (tokenSet_1_.member(cached_LA2)) && (tokenSet_2_.member(LA(3)))))
 		{
-			int _m50 = mark();
-			synPredMatched50 = true;
+			int _m75 = mark();
+			synPredMatched75 = true;
 			inputState.guessing++;
 			try {
 				{
@@ -1231,12 +1595,12 @@ _loop24_breakloop:					;
 			}
 			catch (RecognitionException)
 			{
-				synPredMatched50 = false;
+				synPredMatched75 = false;
 			}
-			rewind(_m50);
+			rewind(_m75);
 			inputState.guessing--;
 		}
-		if ( synPredMatched50 )
+		if ( synPredMatched75 )
 		{
 			mRE_LITERAL(false);
 			if (0==inputState.guessing)
@@ -1280,7 +1644,7 @@ _loop24_breakloop:					;
 		
 		match('/');
 		{ // ( ... )+
-			int _cnt85=0;
+			int _cnt111=0;
 			for (;;)
 			{
 				if ((tokenSet_1_.member(cached_LA1)))
@@ -1289,12 +1653,12 @@ _loop24_breakloop:					;
 				}
 				else
 				{
-					if (_cnt85 >= 1) { goto _loop85_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt111 >= 1) { goto _loop111_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				
-				_cnt85++;
+				_cnt111++;
 			}
-_loop85_breakloop:			;
+_loop111_breakloop:			;
 		}    // ( ... )+
 		match('/');
 		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
@@ -1389,6 +1753,20 @@ _loop85_breakloop:			;
 		returnToken_ = _token;
 	}
 	
+	public void mONES_COMPLEMENT(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
+{
+		int _ttype; IToken _token=null; int _begin=text.Length;
+		_ttype = ONES_COMPLEMENT;
+		
+		match('~');
+		if (_createToken && (null == _token) && (_ttype != Token.SKIP))
+		{
+			_token = makeToken(_ttype);
+			_token.setText(text.ToString(_begin, text.Length-_begin));
+		}
+		returnToken_ = _token;
+	}
+	
 	public void mCMP_OPERATOR(bool _createToken) //throws RecognitionException, CharStreamException, TokenStreamException
 {
 		int _ttype; IToken _token=null; int _begin=text.Length;
@@ -1478,7 +1856,7 @@ _loop85_breakloop:			;
 		_ttype = WS;
 		
 		{ // ( ... )+
-			int _cnt64=0;
+			int _cnt90=0;
 			for (;;)
 			{
 				switch ( cached_LA1 )
@@ -1513,12 +1891,12 @@ _loop85_breakloop:			;
 				}
 				default:
 				{
-					if (_cnt64 >= 1) { goto _loop64_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+					if (_cnt90 >= 1) { goto _loop90_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 				}
 				break; }
-				_cnt64++;
+				_cnt90++;
 			}
-_loop64_breakloop:			;
+_loop90_breakloop:			;
 		}    // ( ... )+
 		if (0==inputState.guessing)
 		{
@@ -1555,11 +1933,11 @@ _loop64_breakloop:			;
 				}
 				else
 				{
-					goto _loop68_breakloop;
+					goto _loop94_breakloop;
 				}
 				
 			}
-_loop68_breakloop:			;
+_loop94_breakloop:			;
 		}    // ( ... )*
 		_saveIndex = text.Length;
 		match('\'');
@@ -1894,7 +2272,7 @@ _loop68_breakloop:			;
 			case '8':  case '9':
 			{
 				{ // ( ... )+
-					int _cnt91=0;
+					int _cnt117=0;
 					for (;;)
 					{
 						if (((cached_LA1 >= '0' && cached_LA1 <= '9')) && (tokenSet_2_.member(cached_LA2)) && (true))
@@ -1903,29 +2281,29 @@ _loop68_breakloop:			;
 						}
 						else
 						{
-							if (_cnt91 >= 1) { goto _loop91_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
+							if (_cnt117 >= 1) { goto _loop117_breakloop; } else { throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());; }
 						}
 						
-						_cnt91++;
+						_cnt117++;
 					}
-_loop91_breakloop:					;
+_loop117_breakloop:					;
 				}    // ( ... )+
 				break;
 			}
 			case 'x':
 			{
 				match('x');
-				mDIGIT(false);
-				mDIGIT(false);
+				mHEXDIGIT(false);
+				mHEXDIGIT(false);
 				break;
 			}
 			case 'u':
 			{
 				match('u');
-				mDIGIT(false);
-				mDIGIT(false);
-				mDIGIT(false);
-				mDIGIT(false);
+				mHEXDIGIT(false);
+				mHEXDIGIT(false);
+				mHEXDIGIT(false);
+				mHEXDIGIT(false);
 				break;
 			}
 			case '\\':
@@ -2058,6 +2436,16 @@ _loop91_breakloop:					;
 				match(']');
 				break;
 			}
+			case '{':
+			{
+				match('{');
+				break;
+			}
+			case '}':
+			{
+				match('}');
+				break;
+			}
 			default:
 			{
 				throw new NoViableAltForCharException(cached_LA1, getFilename(), getLine(), getColumn());
