@@ -302,6 +302,19 @@ namespace Boo.Lang.Parser.Tests
 			Assert.AreEqual("/foo\\(bar\\)/", ((RELiteralExpression)ae.Right).Value);
 		}
 
+		[Test]
+		public void TestRELiteral3()
+		{
+			Boo.Lang.Compiler.Ast.Module module = ParseTestCase("re_literal_3.boo");
+			
+			StatementCollection stmts = module.Globals.Statements;
+			Assert.AreEqual(2, stmts.Count);
+			
+			BinaryExpression ae = (BinaryExpression)((ExpressionStatement)stmts[0]).Expression;
+			Assert.AreEqual(BinaryOperatorType.Assign, ae.Operator);
+			Assert.AreEqual("/\\x2f\\u002f/", ((RELiteralExpression)ae.Right).Value);
+		}
+
 		[Test]
 		public void TestIfElse1()
 		{
